import json
import logging
from collections.abc import Generator
from importlib.metadata import version
from pathlib import Path
from typing import Annotated

import typer
from rich import print  # noqa: A004

from rh_aws_saml_login import core
from rh_aws_saml_login.utils import blend_text, enable_requests_logging

app = typer.Typer(rich_markup_mode="rich")
BANNER = r"""
         __                                                         __      __            _
   _____/ /_        ____ __      _______      _________ _____ ___  / /     / /___  ____ _(_)___
  / ___/ __ \______/ __ `/ | /| / / ___/_____/ ___/ __ `/ __ `__ \/ /_____/ / __ \/ __ `/ / __ \
 / /  / / / /_____/ /_/ /| |/ |/ (__  )_____(__  ) /_/ / / / / / / /_____/ / /_/ / /_/ / / / / /
/_/  /_/ /_/      \__,_/ |__/|__/____/     /____/\__,_/_/ /_/ /_/_/     /_/\____/\__, /_/_/ /_/
                                                                                /____/
"""
APP_NAME = "rh-aws-saml-login"
APP_DIR = Path(typer.get_app_dir(APP_NAME))
APP_DIR.mkdir(exist_ok=True)
ACCOUNT_CACHE = APP_DIR / "account_cache.json"


def write_accounts_cache(accounts: list[str]) -> None:
    """Write accounts cache to disk."""
    json.dump(accounts, ACCOUNT_CACHE.open("w"))


def read_accounts_cache() -> list[str]:
    """Read accounts cache from disk."""
    if ACCOUNT_CACHE.exists():
        return json.load(ACCOUNT_CACHE.open())
    return []


def complete_account(ctx: typer.Context, incomplete: str) -> Generator[str, None, None]:  # noqa: ARG001
    for name in read_accounts_cache():
        if name.startswith(incomplete):
            yield name


def version_callback(value: bool) -> None:  # noqa: FBT001
    if value:
        print(f"Version: {version(APP_NAME)}")
        raise typer.Exit


@app.command(epilog="Made with [red]:heart:[/] by [blue]https://github.com/app-sre[/]")
def cli(  # noqa: PLR0913
    account_name: Annotated[
        str | None,
        typer.Argument(
            help="AWS account name. '.' as shortcut to use $AWS_ACCOUNT_NAME.",
            autocompletion=complete_account,
        ),
    ] = None,
    region: Annotated[str, typer.Option(help="AWS region")] = "us-east-1",
    saml_url: Annotated[
        str,
        typer.Option(
            help="SAML URL",
        ),
    ] = "https://auth.redhat.com/auth/realms/EmployeeIDP/protocol/saml/clients/itaws",
    open_command: Annotated[
        str,
        typer.Option(
            help="Command to open the browser (e.g. 'xdg-open' on Linux)",
            envvar="RH_AWS_SAML_LOGIN_OPEN_COMMAND",
        ),
    ] = "open",
    *,
    debug: Annotated[bool, typer.Option(help="Enable debug mode")] = False,
    console: Annotated[
        bool,
        typer.Option(help="Open the AWS console in browser instead of a local shell"),
    ] = False,
    display_banner: Annotated[bool, typer.Option(help="Display a shiny banner")] = True,
    version: Annotated[  # noqa: ARG001
        bool | None, typer.Option("--version", callback=version_callback)
    ] = None,
) -> None:
    """Login to AWS using SAML."""
    logging.basicConfig(
        level=logging.INFO if not debug else logging.DEBUG, format="%(message)s"
    )
    if display_banner:
        print(blend_text(BANNER, (32, 32, 255), (255, 32, 255)))
    if debug:
        enable_requests_logging()
    accounts = core.main(
        account_name=account_name,
        region=region,
        console=console,
        saml_url=saml_url,
        open_command=open_command,
    )
    write_accounts_cache(accounts)


if __name__ == "__main__":
    app()



from collections.abc import Callable
from pathlib import Path

import pytest
from requests_mock import Mocker as RequestsMocker

from rh_aws_saml_login.core import (
    AwsAccount,
    get_aws_accounts,
    get_saml_auth,
    select_aws_account,
)


@pytest.fixture
def fx() -> Callable:
    """Return a function to read fixture files."""

    def _fx(filename: str) -> str:
        return Path(f"tests/fixtures/{filename}").read_text(encoding="locale")

    return _fx


@pytest.fixture
def accounts() -> list[AwsAccount]:
    """Return a list of AwsAccount objects."""
    return [
        AwsAccount(
            name="account-1",
            uid="1234567890",
            role_name="admin-role",
            role_arn="arn:aws:iam::1234567890:role/admin-role",
        ),
        AwsAccount(
            name="account-1",
            uid="1234567890",
            role_name="read-only",
            role_arn="arn:aws:iam::1234567890:role/read-only",
        ),
        AwsAccount(
            name="account-2",
            uid="5432167890",
            role_name="admin-role",
            role_arn="arn:aws:iam::5432167890:role/admin-role",
        ),
        AwsAccount(
            name="987654321",
            uid="987654321",
            role_name="987654321-admin",
            role_arn="arn:aws:iam::987654321:role/987654321-admin",
        ),
    ]


def test_get_saml_auth(requests_mock: RequestsMocker, fx: Callable) -> None:
    """Test get_saml_auth."""
    url = "https://example.com"

    requests_mock.get(url, text=fx("saml.html"))
    aws_url, saml_token = get_saml_auth(url)
    assert aws_url == "http://localhost:8000/aws-sso.html"
    assert saml_token == "fake-saml-token"  # noqa: S105


def test_get_aws_accounts(
    requests_mock: RequestsMocker, fx: Callable, accounts: list[AwsAccount]
) -> None:
    """Test get_aws_accounts."""
    url = "https://example.com"
    saml_token = "fake-saml"  # noqa: S105
    requests_mock.post(url, text=fx("aws-sso.html"))

    assert get_aws_accounts(url, saml_token) == accounts


def test_select_aws_account(accounts: list[AwsAccount]) -> None:
    """Test select_aws_account."""
    account = select_aws_account(accounts, "account-1", "read-only")
    assert account == accounts[1]

    account = select_aws_account(accounts, "account-1")
    assert account == accounts[0]

    account = select_aws_account(accounts, "account-1", "non-existent-role")
    assert not account
    account = select_aws_account(accounts, "non-existent-account")
    assert not account
    account = select_aws_account(accounts, "non-existent-account", "non-existent-role")
    assert not account
